"""
"chesney" initialisation.
"""

import logging

import boto3
from ec2_metadata import ec2_metadata
from requests.exceptions import ConnectTimeout

from chesney.exceptions import ClientError
from chesney.exceptions import NotRunningOnEC2Error
from chesney.exceptions import ParentAutoScalingGroupIDNotFound

LOG = logging.getLogger(__name__)


def get_auto_scaling_group_instances(auto_scaling_group_name, region):
    """
    Get a list of the IDs of the EC2 instances in this auto-scaling group.

    Args:
        auto_scaling_group_name (str): Name of the auto-scaling group.
        region (str):                  AWS region.

    Returns:
        str[]: List of EC2 instance IDs.
    """

    LOG.info('Getting EC2 instances within auto-scaling group "%s"...',
             auto_scaling_group_name)

    client = boto3.client('autoscaling', region_name=region)

    response = client.describe_auto_scaling_groups(
        AutoScalingGroupNames=[
            auto_scaling_group_name
        ])

    auto_scaling_groups = response['AutoScalingGroups']
    auto_scaling_group = auto_scaling_groups[0]
    instances = auto_scaling_group['Instances']
    instance_ids = []

    for instance in instances:
        instance_id = instance['InstanceId']
        LOG.info('Found instance ID "%s".', instance_id)
        instance_ids.append(instance_id)

    instance_ids.sort()
    LOG.info('Found instance IDs: %s', instance_ids)
    return instance_ids


def get_instance_id():
    """
    Get this EC2 instance's ID.

    Raises:
        NotRunningOnEC2Error: The script is not running on an EC2 instance.

    Returns:
        str: EC2 instance ID.
    """

    LOG.info('Getting this EC2 instance ID...')

    try:
        instance_id = ec2_metadata.instance_id
        LOG.info('This is EC2 instance ID "%s".', instance_id)
        return instance_id
    except ConnectTimeout:
        raise NotRunningOnEC2Error()
    except OSError:
        raise NotRunningOnEC2Error()


def get_parent_auto_scaling_group_name(instance_id, region):
    """
    Gets the name of the auto-scaling group which this EC2 instance is a
    member of.

    Args:
        instance_id (str): EC2 instance ID.
        region (str): AWS region.

    Raises:
        ClientError: A boto3 exception.
        ParentAutoScalingGroupIDNotFound: The name of the auto-scaling group
                                          could not be found.

    Returns:
        str: Name of the auto-scaling group.
    """

    LOG.info('Getting the name of the parent auto-scaling group of instance '
             '"%s"...',
             instance_id)

    client = boto3.client('ec2', region_name=region)

    try:
        response = client.describe_instances(InstanceIds=[instance_id])
    except client.exceptions.ClientError as error:
        raise ClientError(error)

    reservations = response['Reservations']
    reservation = reservations[0]
    instances = reservation['Instances']
    instance = instances[0]
    tags = instance['Tags']

    for tag in tags:
        if tag['Key'] == 'aws:autoscaling:groupName':
            auto_scaling_group_name = tag['Value']
            LOG.info('Auto-scaling group name is "%s".',
                     auto_scaling_group_name)
            return auto_scaling_group_name

    LOG.error('Did not find "aws:autoscaling:groupName" tag in: %s', tags)
    raise ParentAutoScalingGroupIDNotFound()


def get_region():
    """
    Gets the AWS region in which this EC2 instance resides.

    Raises:
        NotRunningOnEC2Error: The script is not running on an EC2 instance.

    Returns:
        str: Region.
    """

    LOG.info('Getting this EC2 instance\'s region...')

    try:
        region = ec2_metadata.region
        LOG.info('This is region "%s".', region)
        return region
    except ConnectTimeout:
        raise NotRunningOnEC2Error()
    except OSError:
        raise NotRunningOnEC2Error()


def is_appointed():
    """
    Indicates whether or not this EC2 instance is the one and only.

    Returns:
        bool: Is appointed.
    """

    region = get_region()
    instance_id = get_instance_id()

    auto_scaling_group_name = get_parent_auto_scaling_group_name(
        instance_id=instance_id,
        region=region)

    instance_ids = get_auto_scaling_group_instances(
        auto_scaling_group_name=auto_scaling_group_name,
        region=region)

    if len(instance_ids) == 1:
        LOG.info('This is the only EC2 instance in the auto-scaling group, '
                 'so it is appointed.')
        return True

    if instance_ids[0] != instance_id:
        LOG.info('This instance ID is not alphabetically first, so it is not '
                 'appointed.')
        return False

    LOG.info('This instance ID is alphabetically first, so it is appointed.')
    return True
